﻿// 2017/6/24 Viktor Fukala
// Soutěž v programování - UHK - Hradec Králové

using System;
using System.IO;
using System.Net;
using System.Globalization;

namespace Autopilot
{
    class Program
    {
        // výstup logu do souboru
        static StreamWriter writer;

        static void Main(string[] args)
        {
            // log do souboru
            FileStream outStream = new FileStream("Log.txt", FileMode.Create);
            writer = new StreamWriter(outStream);

            string server = args[0];
            double limit = double.Parse(args[1]);
            FlyState state = new FlyState(server, "start");

            // stavy motorů
            bool up = false, left = false, right = false;

            // řídící smyčka
            while ((state = new FlyState(server, "stav?id=" + state.id +
                "&nahoru=" + BoolChar(up) +
                "&doleva=" + BoolChar(left) +
                "&doprava=" + BoolChar(right))).done == null)
            {
                up = left = right = false;
                
                double landx = LandingX(state.x, state.y, state.vx, state.vy);

                // zapnutí motorů tak, aby se modul trefil na plochu
                if (landx < -state.land)
                {
                    right = true;
                    if (LandingXRight(state.x, state.y, state.vx, state.vy) < -state.land)
                    {
                        up = true;
                    }
                }
                else if (landx > state.land)
                {
                    left = true;
                    if (LandingXLeft(state.x, state.y, state.vx, state.vy) > state.land)
                    {
                        up = true;
                    }
                }
                else
                {
                    // úprava rychlosti x pod 5
                    if (state.vx > 5)
                    {
                        right = true;
                    }
                    else if (state.vx < -5)
                    {
                        left = true;
                    }
                }

                // úprava dopadové rychlost y pod 5
                double landTime = PosQuadratic(-3.5, state.vy, state.y);
                if ((landTime * 3 + state.vy) < -4.7)
                {
                    up = true;
                }

                // úprava rychlostí podle zadaného limitu
                if (limit != 0)
                {
                    if (state.vx > limit * 0.95)
                    {
                        right = false;
                        left = true;
                    }
                    else if (-state.vx > limit * 0.95)
                    {
                        right = true;
                        left = false;
                    }
                    if (state.vy > limit * 0.95)
                    {
                        up = false;
                    }
                    else if (-state.vy > limit * 0.95)
                    {
                        up = true;
                    }
                }
                // vypsání lodu do console a do souboru
                OutLine("");
                OutLine("Landing: " + landx.ToString());
                OutLine("Landing radius:" + state.land.ToString());
                OutLine("x = " + state.x.ToString() + ", y = " + state.y.ToString());
                OutLine("vx = " + state.vx.ToString() + ", vy = " + state.vy.ToString());
                OutLine("Fuel: " + state.p.ToString());
                OutLine("Motory nahoru, doleva, doprava:" + BoolChar(up) + BoolChar(left) + BoolChar(right));
            }
            OutLine(state.done);
            Console.ReadLine();

            // ukončení výstupu do souboru
            writer.Close();
            outStream.Close();
        }

        static char BoolChar(bool b)
        {
            return b ? '1' : '0';
        }

        static void OutLine(string line)
        {
            Console.WriteLine(line);
            writer.WriteLine(line);
        }

        // vrátí kladné řešení kvadratické rovnice
        static double PosQuadratic(double a, double b, double c)
        {
            double d = b * b - 4 * a * c;
            double v = -b / 2 / a;
            double addSub = Math.Sqrt(d) / 2 / a;
            double sol1 = v - addSub;
            double sol2 = v + addSub;
            if (sol1 > 0)
            {
                return sol1;
            }
            else if (sol2 > 0)
            {
                return sol2;
            }
            throw new Exception("No positive solution.");
        }

        // vrací x přistání, kdyby byly všechny motory vypnuté
        static double LandingX(double x, double y, double vx, double vy)
        {
            double t = PosQuadratic(-3.5, vy, y);
            return x + vx * t;
        }

        // vrací x přistání, kdyby byl zapnutý jen motor doprava
        static double LandingXRight(double x, double y, double vx, double vy)
        {
            double t = PosQuadratic(-3.5, vy, y);
            return x + vx * t + 3 * t * t;
        }

        // vrací x přistání, kdyby byl zapnutý jen motor doleva
        static double LandingXLeft(double x, double y, double vx, double vy)
        {
            double t = PosQuadratic(-3.5, vy, y);
            return x + vx * t - 3 * t * t;
        }
    }

    class FlyState
    {
        public int id;
        public double x, y, vx, vy, p, land;
        public string done;

        public FlyState(string server, string request)
        {
            string complete = "http://" + server + "/" + request;
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(complete);
            HttpWebResponse res = (HttpWebResponse)req.GetResponse();
            Stream str = res.GetResponseStream();
            StreamReader reader = new StreamReader(str);
            string line = reader.ReadLine();
            if (!int.TryParse(line, out this.id))
            {
                this.done = line;
                return;
            }
            else
            {
                this.done = null;
            }
            string[] xy = reader.ReadLine().Split(' ');
            this.x = double.Parse(xy[0], CultureInfo.InvariantCulture);
            this.y = double.Parse(xy[1], CultureInfo.InvariantCulture) - 10;
            string[] vxy = reader.ReadLine().Split(' ');
            this.vx = double.Parse(vxy[0], CultureInfo.InvariantCulture);
            this.vy = double.Parse(vxy[1], CultureInfo.InvariantCulture);
            this.p = double.Parse(reader.ReadLine(), CultureInfo.InvariantCulture);
            string[] lr = reader.ReadLine().Split(' ');
            double l = double.Parse(lr[0], CultureInfo.InvariantCulture);
            double r = double.Parse(lr[1], CultureInfo.InvariantCulture);
            double shift = (l + r) / 2;
            this.land = (r - l) / 2 - 10;
            this.x -= shift;

            if (this.land < 2)
            {
                this.land *= 0.5;
            }
            else
            {
                this.land -= 1;
            }
        }
    }
}
