﻿// 24/6/2017 Viktor Fukala
// Soutěž v programování - UHK - Hradec Králové

using System;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Net;
using System.Windows.Forms;

namespace Realpilot
{
    public partial class Form1 : Form
    {
        FlyState state;

        // čas ukončení posledního požadavku
        long time;

        // stavy motorů
        bool up = false, right = false, left = false;

        double maxFuel = 0;
        bool outOfFuel = false;

        // indikuje, jestli je právě zpracováván požadavek na server
        bool requesting = false;

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            double x = this.CurX(), y = this.CurY();

            // poloha v okně
            int wx = (int)(this.CurX() * 3), wy = 600 - (int)(this.CurY() * 3);

            // modul
            g.FillRectangle(Brushes.Gray, wx - 30, wy - 30, 60, 60);
            
            // palivo
            g.FillRectangle(Brushes.Gold, wx - 15, wy + 15 - (int)(30 * (this.CurFuel() / maxFuel)), 30, (int)(30 * (this.CurFuel() / maxFuel)));

            // motory
            long time = DateTime.Now.Ticks / TimeSpan.TicksPerMillisecond;
            Brush col = (time % 100 < 50) ? Brushes.Yellow : Brushes.Red;
            if (this.up)
            {
                g.FillRectangle(col, wx - 10, wy + 30, 20, 20);
            }
            if (this.left)
            {
                g.FillRectangle(col, wx + 30, wy - 10, 20, 20);
            }
            if (this.right)
            {
                g.FillRectangle(col, wx - 50, wy - 10, 20, 20);
            }

            // přistávací plocha
            g.FillRectangle(Brushes.Black, (int)(this.state.l * 3), 595, (int)(3 * (this.state.r - this.state.l)), 5);
        }

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            (new Form2()).ShowDialog();
            int borx = this.Size.Width - this.ClientSize.Width;
            int bory = this.Size.Height - this.ClientSize.Height;
            this.Size = new Size(borx + 600, bory + 600);
            this.DoubleBuffered = true;
            this.StartConn();
        }

        // začne nové přisání
        void StartConn()
        {
            this.state = new FlyState(Program.server, "start");
            this.time = DateTime.Now.Ticks / TimeSpan.TicksPerMillisecond;
            this.maxFuel = this.state.p;
            up = right = left = false;
            outOfFuel = false;
            requesting = false;
            this.timer1.Start();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            this.label1.Text = "Rychlost x = " + CurVx().ToString();
            this.label2.Text = "Rychlost y = " + CurVy().ToString();
            this.Refresh();
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            if (this.outOfFuel)
            {
                return;
            }
            if (e.KeyCode == Keys.Up)
            {
                this.up = true;
            }
            else if (e.KeyCode == Keys.Right)
            {
                this.right = true;
            }
            else if (e.KeyCode == Keys.Left)
            {
                this.left = true;
            }
            this.RequestServer();
        }

        // aktualizuje informace se serverem
        void RequestServer()
        {
            if (!requesting)
            {
                requesting = true;
            }
            else
            {
                return;
            }
            FlyState newState = new FlyState(Program.server, "stav?id=" + this.state.id +
                "&nahoru=" + BoolChar(this.up) +
                "&doleva=" + BoolChar(this.left) +
                "&doprava=" + BoolChar(this.right));
            if (newState.done != null)
            {
                // přistání skončilo

                this.timer1.Stop();

                // vypsat úspěšnost
                MessageBox.Show(newState.done, "Konec");

                // začít nové přistání
                this.StartConn();
            }
            else
            {
                this.state = newState;
            }
            this.time = DateTime.Now.Ticks / TimeSpan.TicksPerMillisecond;
            requesting = false;
        }

        static char BoolChar(bool b)
        {
            return b ? '1' : '0';
        }

        private void Form1_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Up)
            {
                this.up = false;
            }
            else if (e.KeyCode == Keys.Right)
            {
                this.right = false;
            }
            else if (e.KeyCode == Keys.Left)
            {
                this.left = false;
            }
            this.RequestServer();
        }

        // vrátí čas v sekundách od ukončení posledního požadavku na server
        double TimeSinceRequest()
        {
            return (DateTime.Now.Ticks / TimeSpan.TicksPerMillisecond - time) / 1000d;
        }

        // vypočítá a vrátí aktuální polohu x přistávacího modulu
        double CurX()
        {
            double t = TimeSinceRequest();
            return state.x + state.vx * t + 3 * (BoolInt(this.right) - BoolInt(this.left)) * t * t;
        }

        // vypočítá a vrátí aktuální polohu y přistávácího modulu
        // zkontroluje, jestli už modul není u země
        double CurY()
        {
            double t = TimeSinceRequest();
            double y = state.y + state.vy * t + 1 / 2d * (-7 + 12 * BoolInt(this.up)) * t * t;
            if (y <= 10)
            {
                this.RequestServer();
            }
            return y;
        }

        double CurVx()
        {
            double t = TimeSinceRequest();
            return this.state.vx + 6 * (BoolInt(this.right) - BoolInt(this.left)) * t;
        }

        double CurVy()
        {
            double t = TimeSinceRequest();
            return this.state.vy + (-7 + 12 * BoolInt(this.up)) * t;
        }

        double CurFuel()
        {
            if (outOfFuel)
            {
                return 0;
            }
            double t = TimeSinceRequest();
            double fuel = this.state.p - (BoolInt(this.left) / 2d + BoolInt(this.right) / 2d + BoolInt(this.up)) * t;
            if (fuel <= 0)
            {
                this.up = this.right = this.left = false;
                this.RequestServer();
                this.outOfFuel = true;
                return 0;
            }
            return fuel;
        }

        static int BoolInt(bool b)
        {
            return b ? 1 : 0;
        }
    }

    // tato třída reprezentuje informace od serveru
    class FlyState
    {
        public int id;
        public double x, y, vx, vy, p, l, r;

        // řetezec odpovědi serveru v případě, že modul je u země
        public string done;

        public FlyState(string server, string request)
        {
            // spojení se serverem
            string complete = "http://" + server + "/" + request;
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(complete);
            HttpWebResponse res = (HttpWebResponse)req.GetResponse();
            Stream str = res.GetResponseStream();
            StreamReader reader = new StreamReader(str);

            // parsování odpovědi
            string line = reader.ReadLine();
            if (!int.TryParse(line, out this.id))
            {
                this.done = line;
                return;
            }
            else
            {
                this.done = null;
            }
            string[] xy = reader.ReadLine().Split(' ');
            this.x = double.Parse(xy[0], CultureInfo.InvariantCulture);
            this.y = double.Parse(xy[1], CultureInfo.InvariantCulture);
            string[] vxy = reader.ReadLine().Split(' ');
            this.vx = double.Parse(vxy[0], CultureInfo.InvariantCulture);
            this.vy = double.Parse(vxy[1], CultureInfo.InvariantCulture);
            this.p = double.Parse(reader.ReadLine(), CultureInfo.InvariantCulture);
            string[] lr = reader.ReadLine().Split(' ');
            this.l = double.Parse(lr[0], CultureInfo.InvariantCulture);
            this.r = double.Parse(lr[1], CultureInfo.InvariantCulture);

            // ukončení streamů
            reader.Close();
            str.Close();
        }
    }
}
